<?php
/**
 * This file is part of the Katalysis WordPress plugin.
 *
 * (c) 2017 Katalysis <wpp@katalysis.io>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Katalysis;

require_once __DIR__ . '/utility.php';
require_once __DIR__ . '/http_client.php';
require_once __DIR__ . '/3rd/sodium_compat/autoload.php';

class Backend
{
    private $localSK;
    private $localPK;
    private $api_base;

    public $origin;

    public  function __construct($seed, $api_base)
    {
        $localK = \Sodium\crypto_sign_seed_keypair(pack("H*", $seed));
        $this->localSK = \Sodium\crypto_sign_secretkey($localK);
        $this->localPK = \Sodium\crypto_sign_publickey($localK);
        $this->api_base = $api_base;

        $this->origin = strtoupper(bin2hex($this->localPK));
    }

    private static function get_runtime_data() {
        try {
            $php_version = phpversion();
            $admin_email = get_option('admin_email', 'unknown');
            return json_encode(array(
                'type' => get_option('_katalysis_plugin_type', 'feather'),
                'name' => network_site_url('/'),
                'email' => get_option('admin_email', 'unknown'),
                'versions' => array('php' => "$php_version", 'plugin' => get_option("_katalysis_plugin_version", 'unknown'))
            ), JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
        } catch (\Exception $e) {
           return '';
        }
    }

    public function event_activate() {
        try {
            $message = Backend::get_runtime_data();
            $signature = bin2hex(\Sodium\crypto_sign_detached($message, $this->localSK));
            HTTPClient::postonce("{$this->api_base}/v1/event?ref={$this->origin}&type=feather_activate&signature=$signature", $message, 2);
            Utility::log("FEATHER: activation event sent");
        } catch (\Exception $e) {
            Utility::log("FEATHER: activation event failed - {$e->getMessage()}");
        }
    }

    public function event_deactivate() {
        try {
            $message = Backend::get_runtime_data();
            $signature = bin2hex(\Sodium\crypto_sign_detached($message, $this->localSK));
            HTTPClient::postonce("{$this->api_base}/v1/event?ref={$this->origin}&type=feather_deactivate&signature=$signature", $message, 2);
    
        } catch (\Exception $e) {
            Utility::log("FEATHER: deactivation event failed - {$e->getMessage()}");
        }
    }

    public function duration_list() {
        try {
            return json_decode(HTTPClient::getonce("{$this->api_base}/v1/subscriptions/durations", '', 2), true);
        } catch(\Exception $e) {
            Utility::log("FEATHER: failed to read durations - {$e->getMessage()}");
            return array();
        }
    }

    public function subscription_list() {
        try {
            return json_decode(HTTPClient::getonce("{$this->api_base}/v1/subscriptions/?origin={$this->origin}", '', 2), true);
        } catch(\Exception $e) {
            Utility::log("FEATHER: failed to read subscriptions - {$e->getMessage()}");
            return array();
        }
    }

    public function subscription_save($owner_email, $name, $id, $price, $duration_type, $duration_length) {
        $message = json_encode(array(
            'origin' => array('name' => network_site_url('/'), 'id' => $this->origin),
            'content' => array(
                'name' => $name, 'id' => $id,
                'price' => array('value' => $price, 'unit' => 'EUX'),
                'duration_type' => $duration_type, 'duration_length' => $duration_length
            ),
            'owner' => $owner_email
        ), JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
        $signature = bin2hex(\Sodium\crypto_sign_detached($message, $this->localSK));

        try {
            HTTPClient::postonce("{$this->api_base}/v1/subscriptions/$id?signature=$signature", $message, 5);
        } catch(\Exception $e) {
            Utility::log("FEATHER: failed to save subscription - {$e->getMessage()}");
        }
    }

    public function origin_save($editor_email, $editor_share, $terms_accepted) {
        $message = json_encode(array(
            'name' => network_site_url('/'),
            'ref' => $this->origin,
            'terms_accepted' => $terms_accepted,
            'editor_email' => $editor_email,
            'editor_share' => $editor_share,
            'type' => get_option('_katalysis_plugin_type', 'feather'),
            'options' => ''
        ), JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
        $signature = bin2hex(\Sodium\crypto_sign_detached($message, $this->localSK));

        try {
            HTTPClient::postonce("{$this->api_base}/v1/origin/{$this->origin}?signature=$signature", $message, 5);
        } catch(\Exception $e) {
            Utility::log("FEATHER: failed to save origin - {$e->getMessage()}");
        }
    }

    public function subscription_delete($id) {
        $message = json_encode(array(
            'origin' => array('name' => network_site_url('/'), 'id' => $this->origin),
            'reason' => 'user action'
        ), JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
        $signature = bin2hex(\Sodium\crypto_sign_detached($message, $this->localSK));

        try {
            HTTPClient::deleteonce("{$this->api_base}/v1/subscriptions/$id?signature=$signature", $message, 2);
        } catch(\Exception $e) {
            Utility::log("FEATHER: failed to delete subscription - {$e->getMessage()}");
        }
    }

    public function post_save($name, $id, $edit_url, $view_url, $price, $editor, $editor_share, $author, $author_share) {
        $message = json_encode(array(
            'origin' => array('name' => network_site_url('/'), 'id' => $this->origin),
            'content' => array(
                'name' => $name,
                'edit_url' => $edit_url,
                'view_url' => $view_url,
                'price' => array('value' => $price, 'unit' => 'EUX'),
                'links' => array(),
                'contributors' => array(
                    array('p' =>  $editor, 's' => $editor_share, 't' => 'percentage'),
                    array('p' =>  $author, 's' => $author_share, 't' => 'relative')
                )
            )
        ), JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
        $signature = bin2hex(\Sodium\crypto_sign_detached($message, $this->localSK));

        try {
            HTTPClient::postonce("{$this->api_base}/v1/content/$id?signature=$signature", $message, 20);
            return true;
        } catch(\Exception $e) {
            Utility::log("FEATHER: failed to save post - {$e->getMessage()}");
            return false;
        }
    }

    public function post_readable($id, $participant_id) {
        try {
            HTTPClient::getonce("{$this->api_base}/v1/content/{$id}/readable?participant={$participant_id}", '', 5);
            return true;
        } catch(\Exception $e) {
            //Utility::log("FEATHER: failed to check post readable state - {$e->getMessage()}");
            return false;
        }
    }

    public function get_subscriptions($participant_id) {
        try {
            $message = json_decode(HTTPClient::getonce(
                "{$this->api_base}/v1/participant/{$participant_id}/subscriptions/bought?origin={$this->origin}&active=true", '', 5));
            return $message->items;
        } catch(\Exception $e) {
            //Utility::log("FEATHER: failed to check post readable state - {$e->getMessage()}");
            return array();
        }
    }
}
