<?php

namespace Katalysis;

require_once __DIR__ . '/http_client.php';
require_once __DIR__ . '/utility.php';
require_once __DIR__ . '/backend.php';
require_once __DIR__ . '/html_renderer.php';

class EnvironmentSettings
{
    public $api_root = 'https://api.katalysis.io';
    public $wallet_root = 'https://wallet.katalysis.io';
    public $wallet_uri = 'https://wallet.katalysis.io/v2/client/wallet.js';
    public $wallet_ui_uri = 'https://wallet.katalysis.io/v2/';
}

class PluginSettings
{
    public static $name = "katalysis_api";

    public $default_owner_fee = 10;
    public $default_show_paragraphs = 1;
    public $default_post_price = 69; //cents
    public $subscriptions_enabled = false;
    public $donations_enabled = false;
    public $donations_to_writer = false;
    public $seed = '';
    public $editor_email = '';
    public $environment = 'production';
    public $wallet_id = 0;
    public $terms_accepted = false;
    public $rendering_priority = 1000;
    public $wallet = array();

    public function to_array() {
        return array(
            'owner_default_fee' => $this->default_owner_fee,
            'default_show_paragraphs' => $this->default_show_paragraphs,
            'default_post_price' => $this->default_post_price,
            'local_seed' => $this->seed,
            'editor_email' => $this->editor_email,
            'environment' => $this->environment,
            'wallet' => $this->wallet,
            'wallet_id' => $this->wallet_id,
            'terms_accepted' => $this->terms_accepted,
            'subscriptions_enabled' => $this->subscriptions_enabled,
            'donations_enabled' => $this->donations_enabled,
            'rendering_priority' => $this->rendering_priority,
            'donations_to_writer' => $this->donations_to_writer);
    }

    public static function from_array($data_array) {
        $settings = new PluginSettings();
        $settings->environment = isset($data_array["environment"]) ?
            $data_array["environment"] : 'production';
        $settings->terms_accepted = isset($data_array["terms_accepted"]) ?
            boolval($data_array['terms_accepted']) : false;
        $settings->wallet = isset($data_array["wallet"]) ?
            $data_array["wallet"] : '';
        $settings->wallet_id = isset($data_array["wallet_id"]) ?
            $data_array["wallet_id"] : 0;
        $settings->subscriptions_enabled = isset($data_array["subscriptions_enabled"])
            ? boolval($data_array["subscriptions_enabled"]) : false;
        $settings->donations_enabled = isset($data_array["donations_enabled"])
            ? boolval($data_array["donations_enabled"]) : false;
        $settings->donations_to_writer = isset($data_array["donations_to_writer"])
            ? boolval($data_array["donations_to_writer"]) : false;
        $settings->default_owner_fee = isset($data_array["owner_default_fee"])
            ? floatval($data_array['owner_default_fee']) : 10;
        $settings->default_show_paragraphs = isset($data_array["default_show_paragraphs"])
            ? intval($data_array['default_show_paragraphs']) : 1;
        $settings->editor_email = isset($data_array["editor_email"])
            ? $data_array['editor_email'] : '';
        $settings->default_post_price = isset($data_array["default_post_price"])
            ? intval(floatval($data_array["default_post_price"])) : 69;
        $settings->rendering_priority = isset($data_array["rendering_priority"])
            ? intval(floatval($data_array["rendering_priority"])) : 1000;
        $settings->seed = isset($data_array["local_seed"])
            ? $data_array['local_seed'] : '';
        return $settings;
    }

    public function get_wallet_customizations_as_data_string() {
        $wallet = '';
        foreach ($this->wallet as $key => $value) {
            $wallet .= "data-$key='$value' ";
        }
        return $wallet;
    }
}

class PostSettings
{
    public static $name = "_katalysis_pricing";

    public $price = 0;
    public $paragraphs = 1;
    public $donations_enabled = false;

    public function to_array() {
        return array('price' => $this->price, 'paragraphs' => $this->paragraphs, 'donations_enabled' => $this->donations_enabled);
    }
}

class PluginSettingsManager
{
    private static $instance = NULL;
    private $free_paragraphs_callback = NULL;

    public static $ui_group_general = "katalysis_general";
    public static $ui_group_subscriptions = "katalysis_subscriptions";
    public static $ui_group_development = "katalysis_development";

    public $backend;

    public static function get_instance() {
        if ( self::$instance == NULL ) {
            self::$instance = new self();
        }

        return self::$instance;
    }

    final private function __construct() { 
        try {
            $settings = $this->get_plugin_settings();
        } catch(\Exception $e) {
            $error = $e->getMessage();
            Utility::log("FEATHER: Unable to read plugin settings - $error");
            $settings = new PluginSettings();
        }

        if ($settings->seed == '') {
            mt_srand(microtime(true));
            $randomValue = function_exists('openssl_random_pseudo_bytes')
                ? openssl_random_pseudo_bytes(1024)
                : ((string)mt_rand() . uniqid());
            $settings->seed = Utility::str_to_hex(substr($randomValue, 0, 32)); // 32 bytes
        }

        $environment = $this->get_environment_settings($settings->environment);
        $this->backend = new Backend($settings->seed, $environment->api_root);
        if (!$settings->subscriptions_enabled) {
            $settings->subscriptions_enabled = (sizeof($this->backend->subscription_list()) > 0);
        }
        update_option(PluginSettings::$name, $settings->to_array());
    }

    public function register_free_paragraphs_callback($callback) {
        $this->free_paragraphs_callback = $callback;
    }

    public function echo_plugin_general_settings_view()
    {
?>
        <div class="wrap">
            <h2>Feather by Katalysis - General</h2>
            <hr>
            <form method="post" action="options.php">
                <?php settings_fields(PluginSettingsManager::$ui_group_general); ?>
                <?php do_settings_sections(PluginSettingsManager::$ui_group_general); ?>
                <?php settings_errors(); ?>
            <hr>
                <?php submit_button('Save');?>
            </form>
        </div>
<?php
    }

    public function register_plugin_settings()
    {
        $info_url = plugins_url('/assets/img/info.png', __FILE__);
        add_settings_section(PluginSettingsManager::$ui_group_general, 'General', function($_) {}, PluginSettingsManager::$ui_group_general);
            add_settings_field(
            'terms_accepted', 'I accept <a href="https://katalysis.io/feather/terms/creator.v1.pdf" target="_blank">Terms and conditions</a>',
            function() {
                $option = get_option(PluginSettings::$name);
                $value = $option['terms_accepted'] ? 'checked' : '';
                echo "<input type='checkbox' id='katalysis_terms_accepted' class='regular-text' name='terms_accepted' $value />";
            },
            PluginSettingsManager::$ui_group_general, PluginSettingsManager::$ui_group_general);
        add_settings_field(
            'donations_enabled', "<a target='_blank' href='https://katalysis.io/knowledge-base/starten-met-donaties'><img src='$info_url' title='If you check this box, all new posts will have a donation box underneath by default'/></a>Donations enabled by default",
            function() {
                $option = get_option(PluginSettings::$name);
                $value = $option['donations_enabled'] ? 'checked' : '';
                echo "<input type='checkbox' id='katalysis_donations_enabled' class='regular-text' name='donations_enabled' $value />";
            },
            PluginSettingsManager::$ui_group_general, PluginSettingsManager::$ui_group_general);
        add_settings_field(
            'donations_to_writer', "<a target='_blank' href='https://katalysis.io/knowledge-base/starten-met-donaties'><img src='$info_url' title='If you check this box, donations to post will go to its author (not editor)'/></a>Post donations go to author",
            function() {
                $option = get_option(PluginSettings::$name);
                $value = $option['donations_to_writer'] ? 'checked' : '';
                echo "<input type='checkbox' id='katalysis_donations_to_writer' class='regular-text' name='donations_to_writer' $value />";
            },
            PluginSettingsManager::$ui_group_general, PluginSettingsManager::$ui_group_general);
        add_settings_field(
            'editor_email', " <a target='_blank' href='https://katalysis.io/knowledge-base/set-up-feather'><img src='$info_url' title='Email, which will receive a share of the post sells'/></a> Editor Email",
            function() {
                $option = get_option(PluginSettings::$name);
                $value = $option['editor_email'];
                echo "<input type='text' id='katalysis_editor_email' class='regular-text' name='editor_email' size='40' value='$value' />";
            },
            PluginSettingsManager::$ui_group_general, PluginSettingsManager::$ui_group_general);
        add_settings_field(
            'owner_default_fee', "<a target='_blank' href='https://katalysis.io/knowledge-base/set-up-feather'><img src='$info_url' title='% of the post price, which will be transferred to the editor'/></a> Editor Share (%)",
            function() {
                $option = get_option(PluginSettings::$name);
                $value = $option['owner_default_fee'];
                echo "<input type='text' id='katalysis_owner_default_fee' class='regular-text' name='owner_default_fee' size='40' value='$value' />";
            },
            PluginSettingsManager::$ui_group_general, PluginSettingsManager::$ui_group_general);
        add_settings_field(
            'default_post_price', "<a target='_blank' href='https://katalysis.io/knowledge-base/set-up-feather'><img src='$info_url' title=' To make things easy for you you can specify your default price here per article. This means that automatically all post are specified at this default price unless you change it when publishing the article'/></a> Default post price",
            function() {
                try {
                    $option = get_option(PluginSettings::$name);
                    $value = floatval($option['default_post_price']) / 100.0;
                } catch(\Exception $e) {
                    $value = PluginSettings()->default_post_price;
                }
                echo "<input type='text' id='katalysis_default_post_price' class='regular-text' name='default_post_price' size='40' value='$value' />";
            },
            PluginSettingsManager::$ui_group_general, PluginSettingsManager::$ui_group_general);
        add_settings_field(
            'default_show_paragraphs', "<a target='_blank' href='https://katalysis.io/knowledge-base/set-up-feather'><img src='$info_url' title='To make things easy for you you can specify your default teaser paragraphs here per article. This means that automatically all post show these specified teaser paragraphs. You can manually adjust this per post when publishing the article'/></a> Default number of paragraphs to show",
            function() {
                $option = get_option(PluginSettings::$name);
                $value =$option['default_show_paragraphs'];
                if (!isset(PluginSettingsManager::$instance->free_paragraphs_callback)) {
                    echo "<input type='text' id='katalysis_default_show_paragraphs' class='regular-text' name='default_show_paragraphs' size='40' value='$value' />";
                } else {
                    echo "Inherited from the parent plugin";
                    echo "<input type='hidden' id='katalysis_default_show_paragraphs' class='regular-text' name='default_show_paragraphs' size='40' value='$value' />";
                }
            },
            PluginSettingsManager::$ui_group_general, PluginSettingsManager::$ui_group_general);
        add_settings_field(
                'rendering_priority', "<a target='_blank' href='https://katalysis.io/knowledge-base/set-up-feather'><img src='$info_url' title='Allows you to set the rendering priority of the Wallet. This is useful if you use other plugins with non standard rendering priority such as Jetpack.'/></a> Rendering priority",
                function() {
                    $option = get_option(PluginSettings::$name);
                    $value =$option['rendering_priority'];
                    echo "<input type='text' id='katalysis_rendering_priority' class='regular-text' name='rendering_priority' size='40' value='$value' />";
                },
                PluginSettingsManager::$ui_group_general, PluginSettingsManager::$ui_group_general);
        add_settings_field(
            'wallet', "<a target='_blank' href='https://katalysis.io/knowledge-base/wallet-customisation'><img src='$info_url' title='Feather wallet provides several endpoints for customisation. Click this link to learn more about it'/></a> Wallet customization",
            function() {
                $option = get_option(PluginSettings::$name);
                $value = PluginSettingsManager::wallet_parameters_to_string($option['wallet']);
                echo "<textarea id='katalysis_wallet' class='regular-text' name='wallet' wrap='soft' rows='7' style='width:100%'>$value</textarea>";
            },
            PluginSettingsManager::$ui_group_general, PluginSettingsManager::$ui_group_general);
        add_settings_field(
            'donation_page_url', "<a target='_blank' href='https://katalysis.io/knowledge-base/'><img src='$info_url' title='Embedded html for custom subscription/donation page'></a>Embedded donations html",
            function() {
                $settings = $this->get_plugin_settings();
                $donations_to_writer = $settings->donations_to_writer ? 'true' : 'false';
                $html = HtmlRenderer::wallet_page_html(
                    get_option('_katalysis_plugin_type', 'feather'),
                    site_url(),
                    $this->backend->origin,
                    " data-display-donations='true' data-donations-to-writer='$donations_to_writer' data-display-subscriptions-menu='false'",
                    $settings->get_wallet_customizations_as_data_string());
                echo "Embed donations ui on your custom page by pasting html below into the appropriate location on your page";
                echo "<textarea class='regular-text' wrap='soft' rows='10' style='width:100%'>$html</textarea>";
            },
            PluginSettingsManager::$ui_group_general, PluginSettingsManager::$ui_group_general);
        add_settings_field(
            'subscription_page_url', "<a target='_blank' href='https://katalysis.io/knowledge-base/'><img src='$info_url' title='Embedded html for custom subscription/donation page'></a>Embedded subscriptions html",
            function() {
                $settings = $this->get_plugin_settings();
                $html = HtmlRenderer::wallet_page_html(
                    get_option('_katalysis_plugin_type', 'feather'),
                    site_url(),
                    $this->backend->origin,
                    " data-display-donations='false' data-display-subscriptions='true' data-display-subscriptions-menu='true' ",
                    $settings->get_wallet_customizations_as_data_string());
                echo "Embed subscriptions ui on your custom page by pasting html below into the appropriate location on your page";
                echo "<textarea class='regular-text' wrap='soft' rows='10' style='width:100%'>$html</textarea>";
            },
            PluginSettingsManager::$ui_group_general, PluginSettingsManager::$ui_group_general);
        register_setting(
            PluginSettingsManager::$ui_group_general,
            PluginSettingsManager::$ui_group_general,
            function($_)
            {
                $http_post = $_POST;
                if (!isset($http_post['editor_email'])) {
                    add_settings_error('editor_email', 'editor_email', '<span class="katalysis-settings-error">Please enter an Editor email.</span>');
                }
                if (!isset($http_post['owner_default_fee']) || floatval($http_post['owner_default_fee']) <0 || floatval($http_post['owner_default_fee']) > 100) {
                    add_settings_error('owner_default_fee', 'owner_default_fee', '<span class="katalysis-settings-error">Please enter a default percentage share for the Editor (0-100).</span>');
                }
                if (!isset(PluginSettingsManager::$instance->free_paragraphs_callback))
                {
                    if (!isset($http_post['default_show_paragraphs']) || intval($http_post['default_show_paragraphs']) <= 0) {
                        add_settings_error('default_show_paragraphs', 'default_show_paragraphs', '<span class="katalysis-settings-error">Please enter a default number of paragraphs to show for paid articles.</span>');
                    }
                }

                if (!isset($http_post['default_post_price']) || floatval($http_post['default_post_price']) <=0 ) {
                    add_settings_error(
                        'default_post_price', 'default_post_price',
                        '<span class="katalysis-settings-error">Please enter a default price for the post(euro).</span>');
                }

                $settings = $this->get_plugin_settings();
                $settings->terms_accepted = Utility::try_parse_bool($http_post, 'terms_accepted', false);
                $settings->donations_enabled = Utility::try_parse_bool($http_post, 'donations_enabled', false);
                $settings->donations_to_writer = Utility::try_parse_bool($http_post, 'donations_to_writer', false);
                $settings->wallet = PluginSettingsManager::wallet_parameters_from_string($http_post['wallet']);
                $settings->editor_email = strip_tags(stripslashes($http_post['editor_email']));
                $settings->default_owner_fee = Utility::try_parse_float($http_post, 'owner_default_fee', $settings->default_owner_fee);
                if (!isset(PluginSettingsManager::$instance->free_paragraphs_callback)) {
                    $settings->default_show_paragraphs = intval($http_post['default_show_paragraphs']);
                }
                try {
                    $settings->default_post_price = floatval($http_post['default_post_price']) * 100;
                } catch(\Exception $e) {
                    // using default
                }

                if (!isset($http_post['rendering_priority']) || intval($http_post['rendering_priority']) <0) {
                    $settings->rendering_priority = 1000;
                } else {
                    $settings->rendering_priority = $http_post['rendering_priority'];
                }

                $this->backend->origin_save(
                    $settings->editor_email, $settings->default_owner_fee, $settings->terms_accepted);
                update_option(PluginSettings::$name, $settings->to_array());
    
                return array();
            });

        register_setting(
            PluginSettingsManager::$ui_group_subscriptions,
            PluginSettingsManager::$ui_group_subscriptions,
            function($_)
            {
                $http_post = $_POST;
                $settings = $this->get_plugin_settings();

                $environment = $this->get_environment_settings($settings->environment);
                if (is_null($environment)) {
                    $error = $e->getMessage();
                    Utility::log("FEATHER: Failed to save subscription settings - unable to load environment");
                    return array();
                }

                $action = ''; $id = '';
                $name_field = 'name-new'; $price_field = 'price-new';
                $type_field = 'type-new'; $len_field = 'len-new';
                if (!isset($http_post['new'])) {
                    foreach ($http_post as $key => $value) {
                        if (strpos($key, 'update-') === 0) {
                            $action = 'save'; $id = substr($key, 7);
                            $name_field = "name-$id"; $price_field = "price-$id";
                            $type_field = "type-$id"; $len_field = "len-$id"; 
                            break;
                        } else if (strpos($key, 'delete-') === 0) {
                            $action = 'delete'; $id = substr($key, 7);
                            break;
                        }
                    }
                } else {
                    $action = 'save';
                }

                $message = '';
                if ($action == 'save') {
                    $price = Utility::try_parse_float($http_post, $price_field, 0.0) * 100; // price is stored in cents
                    $length = Utility::try_parse_int($http_post, $len_field, 1.0);
                    $this->backend->subscription_save($settings->editor_email, $http_post[$name_field], $id, $price, $http_post[$type_field], $length);
                } else if ($action == 'delete') {
                    $this->backend->subscription_delete($id);
                }

                return array();
            });

        add_settings_section(PluginSettingsManager::$ui_group_development, 'Development', function($_) {}, PluginSettingsManager::$ui_group_development);
        add_settings_field(
            'environment', 'Environment',
            function() {
                    $environments = json_decode(HTTPClient::getonce('https://env.katalysis.io/', ''), true);
                    $option = get_option(PluginSettings::$name);
                    $value = $option['environment'];
                    echo "<select id='katalysis_environment' name='environment'>";
                    foreach ($environments as $environment) {
                        $is_selected = ($environment['name'] == $value ? 'selected' : '');
                        echo "<option value='{$environment['name']}' $is_selected>{$environment['label']}</option>";
                    }
                    echo "</select>";
            },
            PluginSettingsManager::$ui_group_development, PluginSettingsManager::$ui_group_development);
        add_settings_field(
            'wallet_id', 'Wallet Id',
            function() {
                $option = get_option(PluginSettings::$name);
                $value = $option['wallet_id'];
                echo "<input type='text' id='katalysis_wallet_id' class='regular-text' name='wallet_id' size='40' value='$value' />";
            },
            PluginSettingsManager::$ui_group_development, PluginSettingsManager::$ui_group_development);
        register_setting(
            PluginSettingsManager::$ui_group_development,
            PluginSettingsManager::$ui_group_development,
            function($_)
            {
                $http_post = $_POST;
                $settings = $this->get_plugin_settings();
                if (!isset($http_post['environment'])) {
                    add_settings_error('environment', 'environment', '<span class="katalysis-settings-error">Please select an environment.</span>');
                } else {
                    $settings->environment = strip_tags(stripslashes($http_post['environment']));
                }

                if (isset($http_post['wallet_id'])) {
                    $settings->wallet_id = intval($http_post['wallet_id']);
                } else {
                    $settings->wallet_id = 0;
                }
 
                $environment = $this->get_environment_settings($settings->environment);
                $this->backend = new Backend($settings->seed, $environment->api_root);
                $settings->subscriptions_enabled = (sizeof($this->backend->subscription_list()) > 0);
                update_option(PluginSettings::$name, $settings->to_array());
 
                return array();
            });
    }

    public function echo_plugin_subscriptions_settings_view()
    {
        $settings = $this->get_plugin_settings();
        $environment = $this->get_environment_settings($settings->environment);
        if (is_null($environment)) {
            echo "Unable to read server settings - please contact support";
            return;
        }

        $types = $this->backend->duration_list();
        $subscriptions = $this->backend->subscription_list();
        $settings->subscriptions_enabled = (sizeof($subscriptions) > 0);
        update_option(PluginSettings::$name, $settings->to_array());
?>
        <div class="wrap">
            <h2>Feather by Katalysis - Subscriptions</h2>
            <hr>
Hier kun je je abonnementen plannen opstellen en wijzigen. Wil je meer leren? Lees dan dit <a href="http://help.katalysis.io/getting-started-with-the-feather-plugin/starten-met-en-het-aanpassen-van-abonnementen" target="_blank">artikel</a><br>
Here you can set up and adjust plans for subscriptions. Do you want to learn more? Read this <a href="https://help.katalysis.io/getting-started-with-the-feather-plugin/customise-subscriptions" target="_blank">article</a>
            <HR>
            <form method="post" action="options.php">
                <?php settings_fields(PluginSettingsManager::$ui_group_subscriptions); ?>
                <table border="1">
                    <tr><th>name</th><th>duration unit</th><th>duration length</th><th>price</th><th></th><th></th>
                    <tr>
                        <td><input type='text' name='name-new' size='20' value='New subscription' /></td>
                        <td><select name='type-new'>
                            <?php foreach ($types as $type) { $is_selected = ($type == 'monthly' ? 'selected' : ''); ?>
                                <option value="<?php echo $type ?>" <?php echo $is_selected ?>><?php echo $type ?></option>";
                            <?php } ?>
                        </td>
                        <td><input type='text' name='len-new' size='10' value='1' /></td>
                        <td><input type='text' name='price-new' size='10' value='10' /></td>
                        <td colspan="2"><input type="submit" name="new" size='10' value="Add new" /></td>
                    </tr>
                    <tr><td colspan="6"><hr/></td></tr>
                    <?php foreach ($subscriptions as $subscription) { $sid = $subscription['id']; ?>
                    <tr>
                        <td><input type='text' name='name-<?php echo $sid ?>' size='20' value='<?php echo $subscription['name'] ?>' /></td>
                        <td>
                            <select name="type-<?php echo $sid ?>">
                            <?php foreach ($types as $type) { $is_selected = ($subscription['duration_type'] == $type ? 'selected' : ''); ?>
                                <option value="<?php echo $type ?>" <?php echo $is_selected ?>><?php echo $type ?></option>";
                            <?php } ?>
                            </select>
                        </td>
                        <td><input type="text" name="len-<?php echo $sid ?>" size="10" value="<?php echo $subscription['duration_length'] ?>" /></td>
                        <td><input type="text" name="price-<?php echo $sid ?>" size="10" value="<?php echo ($subscription['price']['value'] / 100.0) ?>" /></td>
                        <td><input type="submit" name="update-<?php echo $sid ?>" size='10' value="Update" /></td>
                        <td><input type="submit" name="delete-<?php echo $sid ?>" size='10' value="Delete" /></td>
                    </tr>
                    <?php } ?>
                </table>
            </form>
        </div>
<?php
    }

    public function get_environment_settings($environment='')
    {
        try {
            $settings = $this->get_plugin_settings();
            $environment = $settings->environment;
            $wallet_id = $settings->wallet_id;

            $json_configs = array_filter(
                json_decode(HTTPClient::getonce("https://env.katalysis.io", ""), true),
                function($json) use ($environment) { return $json['name'] == $environment; });

            if (count($json_configs) == 0) {
                throw new Exception("FEATHER: Settings for environment '$environment' are not available");
            }

            if (count($json_configs) > 1) {
                throw new Exception("FEATHER: Settings for environment '$environment' are ambiguous");
            }

            $json_config_array = array_values($json_configs);
            $json_config = $json_config_array[0];
            if (count($json_config['wallets']) == 0) {
                throw new Exception("FEATHER: No wallet settings for environment '$environment'");
            }

            $wallet_config = $json_config['wallets'][$wallet_id];

            $settings = new EnvironmentSettings();
            $settings->api_root = $json_config['api_root'];
            $settings->wallet_root = $json_config['wallet_root'];
            $settings->wallet_uri = $wallet_config['wallet'];
            $settings->wallet_ui_uri = $wallet_config['wallet_ui'];
            return $settings;
        }
        catch(\Exception $e) {
            $error = $e->getMessage();
            Utility::log("FEATHER: Unable to read environment settings - $error");
            return new EnvironmentSettings();
        }
    }

    public function get_plugin_settings()
    {
        return PluginSettings::from_array(get_option(PluginSettings::$name));
    }

    public function get_post_settings_view($post_id)
    {
        $content_settings = $this->get_post_settings($post_id);
        $plugin = $this->get_plugin_settings();

        $html_price_value = $content_settings->price / 100;
        $html_donations_enabled = $content_settings->donations_enabled ? 'checked' : '';
        $html = "
<div>
    <p>
        <label for='katalysis_pricing_price'>Price in €</label><br />
        <input type='text' name='katalysis_pricing_price' id='katalysis_pricing_price' value='$html_price_value' />
    </p>";
        if (!isset(PluginSettingsManager::$instance->free_paragraphs_callback))
        {
            $html .= "
    <p>
        <label for='katalysis_show_paragraphs'>Paragraphs to show</label><br />
        <input style='width: 100%' type='text' name='katalysis_show_paragraphs' id='katalysis_show_paragraphs' value='{$content_settings->paragraphs}' />
    </p>";
        }

        $html .= "
    <p>
        <input type='checkbox' name='katalysis_donations_enabled' id='katalysis_donations_enabled' $html_donations_enabled>Enable donations</input>
    </p>";

        return $html .=  "</div>";
    }

    public function save_post_settings($post_id, $http_post)
    {
        $price = Utility::try_parse_float($http_post, 'katalysis_pricing_price', 0.0) * 100; // price is stored in cents
        $num_paragraphs = Utility::try_parse_int($http_post, 'katalysis_show_paragraphs', 0);
        $donations_enabled = isset($http_post['katalysis_donations_enabled']);
        if ($price > 0 && $num_paragraphs <= 0)
        {
            Utility::log("FEATHER: Not saving settings - price=$price, paragraphs=$num_paragraphs");
            return false;
        }

        $plugin = $this->get_plugin_settings();
        $settings = new PostSettings();
        $settings->price = $price;
        $settings->paragraphs = $num_paragraphs;
        $settings->donations_enabled = $donations_enabled;
        update_post_meta($post_id, '_katalysis_pricing', $settings->to_array());
        return true;
    }

    public function get_post_settings($post_id)
    {
        $content_settings = get_post_meta($post_id, '_katalysis_pricing', true);
        $settings = new PostSettings();
        $get_default_free_paragraphs = $this->free_paragraphs_callback;
        $save = false;
        $plugin_settings = $this->get_plugin_settings();
        if ($content_settings !== false)
        {
            if (isset($get_default_free_paragraphs)) {
                $settings->paragraphs = $get_default_free_paragraphs();
            } else if(isset($content_settings['paragraphs'])) {
                $settings->paragraphs = $content_settings['paragraphs'];
            }
        } else {
            if (isset($get_default_free_paragraphs)) {
                $settings->paragraphs = $get_default_free_paragraphs();
            } else if(isset($content_settings['paragraphs'])) {
                $settings->paragraphs = $plugin_settings->default_show_paragraphs;
            }
            $save = true;
        }

        if (isset($content_settings['price'])) {
            $settings->price = intval($content_settings['price']);
        } else {
            $settings->price = $plugin_settings->default_post_price;
        }

        if (isset($content_settings['donations_enabled'])) {
            $settings->donations_enabled = boolval($content_settings['donations_enabled']);
        } else {
            $settings->donations_enabled = $save ? $plugin_settings->donations_enabled : ($settings->price <= 0.01);
        }

        if ($save) {
            update_post_meta($post_id, '_katalysis_pricing', $settings->to_array());
        }
        return $settings;
    }

    public function echo_plugin_development_settings_view()
    {
?>
        <div class="wrap">
            <h2>Feather by Katalysis - Development</h2>
            <hr>
            <form method="post" action="options.php">
                <?php settings_fields(PluginSettingsManager::$ui_group_development); ?>
                <?php do_settings_sections(PluginSettingsManager::$ui_group_development); ?>
                <?php settings_errors(); ?>
            <hr>
                <?php submit_button('Save');?>
            </form>
        </div>
<?php
    }

    private static function wallet_parameters_to_string($input)
    {
        $text = '';
        foreach ($input as $key => $value) {
            $text .= "$key= $value&#13;&#10;";
        }
        return $text;
    }

    private static function wallet_parameters_from_string($input)
    {
        $settings = array();

        // split by newline, split by =
        $rows = explode("\n", $input);
        foreach ($rows as $row)
        {
            if (strlen($row) == 0)
                continue;

            $pos = strrpos($row, '=');
            if ($pos === false) {
                $settings[trim($row)] = '';
            } else {
                $settings[trim(substr($row, 0, $pos))] = trim(substr($row, $pos + 1));
            }
        }

        return $settings;
    }
}